unit nzperformance;

interface
uses windows;

type
  TCPUPerformance = record
    CPUIdle : double;
    CPUUsage: single;
  end;

  TTimeSampleInfo = record
    DeltaTime : double;
    TotalTime : double;
    samples   : integer;
    _Reserved : int64;
  end;

procedure Performance_Load;
procedure Performance_Unload;
procedure Perfomance_GetCPUPerformance(var info:TCPUPerformance);

procedure Performance_GetTimeSample(var ts:TTimeSampleInfo);
procedure Performance_ResetTimeSample(var ts:TTimeSampleInfo);
implementation

const
  SystemBasicInformation = 0;
  SystemPerformanceInformation = 2;
  SystemTimeInformation = 3;


type
  TSystem_Basic_Information =  record
    dwUnknown1: DWORD;
    uKeMaximumIncrement: ULONG;
    uPageSize: ULONG;
    uMmNumberOfPhysicalPages: ULONG;
    uMmLowestPhysicalPage: ULONG;
    uMmHighestPhysicalPage: ULONG;
    uAllocationGranularity: ULONG;
    pLowestUserAddress: Pointer;
    pMmHighestUserAddress: Pointer;
    uKeActiveProcessors: ULONG;
    bKeNumberProcessors: byte;
    bUnknown2: byte;
    wUnknown3: word;
  end;

  TSystem_Time_Information = record
    liKeBootTime: LARGE_INTEGER;
    liKeSystemTime: LARGE_INTEGER;
    liExpTimeZoneBias: LARGE_INTEGER;
    uCurrentTimeZoneId: ULONG;
    dwReserved: DWORD;
  end;

  TSystem_Performance_Information =  record
    liIdleTime: LARGE_INTEGER; {LARGE_INTEGER}
    dwSpare: array[0..75] of DWORD;
  end;



var
  NtQuerySystemInformation: function(infoClass: DWORD;
    buffer: Pointer;
    bufSize: DWORD;
    returnSize: PDWORD): DWORD; stdcall = nil;
  SysBaseInfo: TSystem_Basic_Information;

  _Lasttime : int64;
  _Freq : int64;
  _DeltaTime : Double;

procedure Performance_GetTimeSample(var ts:TTimeSampleInfo);
var t:int64;
begin
  QueryPerformanceCounter(t);
  if _Freq>0 then
    ts.Deltatime := (t - ts._Reserved) / _Freq
  else
    ts.Deltatime := 0;
  ts._Reserved := t;
  inc(ts.samples);
  ts.TotalTime := ts.TotalTime + ts.DeltaTime;
end;

procedure Performance_ResetTimeSample(var ts:TTimeSampleInfo);
begin
  QueryPerformanceCounter(ts._Reserved);
  ts.Deltatime := 0;
  ts.samples := 0;
  ts.TotalTime := 0;
end;

function Performance_GetDeltaTime : double;
begin
  result := _DeltaTime;
end;

procedure Performance_Load;
var
  status: Longint; {long}
begin
  QueryPerformanceFrequency(_Freq);

  if @NtQuerySystemInformation = nil then
    NtQuerySystemInformation := GetProcAddress(GetModuleHandle('ntdll.dll'),
      'NtQuerySystemInformation');

  // get number of processors in the system

  status := NtQuerySystemInformation(SystemBasicInformation, @SysBaseInfo, SizeOf(SysBaseInfo), nil);
  if status <> 0 then Exit;

end;

procedure Performance_Unload;
begin
  NtQuerySystemInformation := nil;
end;

var
  liOldIdleTime: LARGE_INTEGER = ();
  liOldSystemTime: LARGE_INTEGER = ();

function Li2Double(x: LARGE_INTEGER): Double;
begin
  Result := x.QuadPart //;x.HighPart * 4.294967296E9 + x.LowPart
end;

procedure Perfomance_GetCPUPerformance(var info:TCPUPerformance);
var
  status: Longint; {long}
  SysTimeInfo: TSystem_Time_Information;
  SysPerfInfo: TSystem_Performance_Information;
  dbSystemTime: Double;
  dbIdleTime: Double;

begin
    // get new system time
    status := NtQuerySystemInformation(SystemTimeInformation, @SysTimeInfo, SizeOf(SysTimeInfo), 0);
    if status <> 0 then Exit;

    // get new CPU's idle time
    status := NtQuerySystemInformation(SystemPerformanceInformation, @SysPerfInfo, SizeOf(SysPerfInfo), nil);
    if status <> 0 then Exit;

    // if it's a first call - skip it
    if (liOldIdleTime.QuadPart <> 0) then
    begin

      // CurrentValue = NewValue - OldValue
      dbIdleTime := Li2Double(SysPerfInfo.liIdleTime) - Li2Double(liOldIdleTime);
      dbSystemTime := Li2Double(SysTimeInfo.liKeSystemTime) - Li2Double(liOldSystemTime);

      // CurrentCpuIdle = IdleTime / SystemTime
      dbIdleTime := dbIdleTime / dbSystemTime;
      info.CPUIdle := dbIdleTime;

      // CurrentCpuUsage% = 100 - (CurrentCpuIdle * 100) / NumberOfProcessors
      info.CPUUsage :=  1 - dbIdleTime  / SysBaseInfo.bKeNumberProcessors;

      // Show Percentage
    end
    else begin
      info.CPUIdle := 0;
      info.CPUUsage:= 0;
    end;
    
    // store new CPU's idle and system time
    liOldIdleTime := SysPerfInfo.liIdleTime;
    liOldSystemTime := SysTimeInfo.liKeSystemTime;


end;

end.
